<?php

/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <FritzWebering@gmx.de> wrote this file.  As long as you retain this notice
 * you can do whatever you want with this stuff. If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return. F. Webering
 * ----------------------------------------------------------------------------
 */


/** \file
 * This file defines the main procedures needed to use Highslide JS in PmWiki.
 * 
 * Highslide JS is a JavaScript image viewer developed by Torstein Hnsi that
 * expands the images inside the current page. This recipe provides a new
 * markup, namely (:highslide:), that allows you to enable the Highslide
 * functionality in any wiki page you like.
 * 
 * Highslide JS is not included in this package but at the time of writing it
 * can be obtained on: http://highslide.com/
 * 
 * Currently you need to tick the 'unobtrusive' option when using the
 * configurator on the highslide website. Otherwise you may use the 'full'
 * package ($Highslide['Configuration']='full') which contains the unobtrusive
 * option but also lots of other stuff that you probably don't need.
 * 
 * The files 'highslide-full.js' and 'highslide.css' as well as the 'graphics'
 * directory should be put in pub/highslide/'. Additionally you can add a file
 * 'highslide-custom.css' to apply your own styles and override the defaults
 * set in 'highslide.css'.
 * 
 * Then every link in your page that has the attribute rel="highslide" will be
 * opened in a highslide popup by default.
 */


$RecipeInfo['Highslide']['Version'] = '2008-11-27';
$FmtPV['$HighslideVersion'] = "'highslide{$RecipeInfo['ThumbListGallery']['Version']}'";

/*
 * Most basic configurations.
 */
SDVA($Highslide, array(
	'Configuration' => 'full', // specifies which highslide-xyz.js file to use.
	'Presets' => array(),      // specifies the preset-xyz.js files to load.
	'Language' => '',          // specifies the lang-xy.js file to load.
	
	// The URL and the path MUST specify the same directory.
	'Directory' => "$FarmD/pub/highslide",
	'DirectoryUrl' => "$PubDirUrl/highslide",
));
SDVA($Highslide['Files'], array(
	// These values are relative to $Highslide['Directory'] and $Highslide['DirectoryUrl']
	'ScriptFile' => 'highslide-%s.js', // if a config is set, the %s is prepended by a dash '-'
	'PresetFile' => 'preset-%s.js',
	'LanguageFile' => 'lang-%s.js',
	'DefaultCss' => 'highslide.css',
	'CustomCss' => 'highslide-custom.css',
));
SDVA($Highslide['InlineConfig'], array(
	'graphicsDir' => "hs.graphicsDir = '{$Highslide['DirectoryUrl']}/graphics/';",
));

$PageCSSListFmt[$Highslide['Directory'].'/'.$Highslide['Files']['DefaultCss']] =
		$Highslide['DirectoryUrl'].'/'.$Highslide['Files']['DefaultCss'];
$PageCSSListFmt[$Highslide['Directory'].'/'.$Highslide['Files']['CustomCss']] =
		$Highslide['DirectoryUrl'].'/'.$Highslide['Files']['CustomCss'];

// define a new markup that enabled Highslide JS on a page
Markup('highslide', 'block', '/\\(:highslide(?:\\s+(.*?))?:\\)/ei', "MakeHighslideScript(PSS('$1'))");

/**
 * This function returns a bunch of <script> markups that include and configure
 * Highslide JS as specified by the $Highslide array and the markup options.
 *
 * See the documentation on http://www.pmwiki.org/wiki/Cookbook/Highslide for
 * further information on coniguration options.
 */
function MakeHighslideScript($args) {
	global $Highslide, $MessagesFmt;
	
	// Changing the working directory makes using file_exist easier.
	$dir = getcwd();
	if (!chdir($Highslide['Directory'])) return "'''Error:''' Can not access the specified highslide directory.";
	$hsurl = $Highslide['DirectoryUrl'];
	$opt = ParseArgs($args);
	
	// Sanitise and find the main highslide script.
	preg_match('/\w[-.+\w]*/i', IsEnabled($opt['config'], $Highslide['Configuration']), $match);
	$files = array(0 => sprintf($Highslide['Files']['ScriptFile'], $match[0].'.packed'));
	if (!file_exists($files[0])) $files[0] = sprintf($Highslide['Files']['ScriptFile'], $match[0]);
	if (!file_exists($files[0])) $files[0] = 'highslide.js';
	// Sanitise and find the language file.
	if ($lang = IsEnabled($opt['lang'], $Highslide['Language'])) {
		preg_match('/\w[-.\w]*/i', $lang, $match);
		$files[1] = sprintf($Highslide['Files']['LanguageFile'], $match[0]);
		if (!file_exists($files[1])) $MessagesFmt[] = "<p><b>Warning:</b> The file <span style='font-family:monospace;'>{$files[1]}</span> for translating highslide to the language code <span style='font-family:monospace;'>{$match[0]}</span> does not exist.</p>\n";
	}
	// Figure out which presets to load.
	if (!is_array($Highslide['Presets'])) $Highslide['Presets'] = array((string) $Highslide['Presets']);
	$presets = array_unique(array_merge($Highslide['Presets'], explode(',', $opt['presets'])));
	// Sanitise preset names and remove all items that are prefixed with a minus from the list
	$presets = array_diff(preg_grep('/^\w[-.+\w]*$/i', $presets), preg_replace('/^(?:-(\w[-.+\w]*)|.*)$/i','\1', $presets));
	// Convert preset names to file names
	$presets = array_map(create_function('$a', "return sprintf('{$Highslide['Files']['PresetFile']}', \$a);"), $presets);
	// Add the main and language files and then remove all files that do not exist
	$files = array_filter(array_merge($files, $presets), 'file_exists');
	// Notify the user of those
	$diff = array_diff($presets, $files);
	if ($diff) $MessagesFmt[] = '<p><b>Warning:</b> The following files you (or your admin) specified for the <span style=\'font-family:monospace;\'>(:highslide:)</span> markup do not exist: <span style=\'font-family:monospace;\'>'.implode(', ',$diff)."</span></p>\n";
	// Create html markup from all remaining files
	$pattern = "\n	<script type=\"text/javascript\" src=\"$hsurl/%s\"></script>";
	$scripts = vsprintf(str_repeat($pattern,	count($files)), $files);
	// Format the inline configuration statements
	$settings = implode("\n		", (array)$Highslide['InlineConfig']);
	
	chdir($dir);
	return Keep(
	"$scripts
	<script type=\"text/javascript\">
		// \$Highslide['InlineConfig']
		$settings
	</script>");
}
